-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobgetrelatedbyclass.sql, tbsm_base, tbsm_base31e
-- SID: 1.5.4.3
-- Date Modified: 07/09/10 08:52:43
--
-- Module Description
--   Defines the alobGetRelatedByClass stored procedure

Use Object
GO

-------------------------------------------------------------------------
-- alobGetRelatedByClass
PRINT 'Creating procedure alobGetRelatedByClass'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobGetRelatedByClass' AND type = 'P')
  DROP PROCEDURE alobGetRelatedByClass
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobgetrelatedbyclass.sql, tbsm_base, tbsm_base31e
-- SID: 1.5.4.3
-- Date Modified: 07/09/10 08:52:43


CREATE PROCEDURE alobGetRelatedByClass
   @incStart     INTEGER     = 0   -- If 1, then a starting object Pattern indicated
                                   -- that it must be included in paths when analyzing
                                   -- ObjPathCache for descendents.
  
AS

  -- Standard TRANSACTION logic

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN alobGetRelatedByClass
  ELSE
    SAVE TRAN alobGetRelatedByClass

  -- Standard Announcement 
  RAISERROR( 'alobGetRelatedByClass', 0, 1 )
/*
  -- The following tables must pre-exist the call to this stored procedure
  -- And as odd as it is #result drives the population of #match
  CREATE TABLE #resultCriteria  
  CREATE TABLE #data
*/

  -- HACK: This stored procedure may need to be limited to the objects that are related to a particular pattern.
  --       Actually it will need to do both, or handle the 

  -- Cleanup #data
  TRUNCATE TABLE #data

  CREATE TABLE #needed
  (
    iClass         ClassID,
    iObject        ObjID,
    iClassRelated  ClassID,
    iClassStart    ClassID,
    iObjectStart   ObjID,
    iPattern       INTEGER         NOT NULL,
    sNameAttribute NVARCHAR( 256 ) NOT NULL
  )

  CREATE INDEX tneededIndex ON #needed(iClass,iObject,iClassRelated,iClassStart,iObjectStart,iPattern,sNameAttribute)


  -- Create a list of iPatterns that need to be tested
  -- PRINT 'Create a list of iPatterns that need to be tested'
  -- alobListCriteria
  -- iCriteria   iPattern    iPatternRelated 
  -- ----------- ----------- --------------- 
  -- 1           1           2              <- This exists in #resultCriteria
  -- 1           2           any            <- This row will be added to #needed

  --  SELECT r.iClass, r.iObject, c.iPatternRelated, p.sPatternClass, p.sPatternAttribute
  INSERT INTO #needed (   iClass,   iObject, iClassRelated,   iClassStart,   iObjectStart,   iPattern,    sNameAttribute )
  SELECT DISTINCT       r.iClass, r.iObject,      p.iClass, r.iClassStart, r.iObjectStart, p.iPattern,  p.sNameAttribute 
  FROM 	     #resultCriteria          AS r 
  INNER JOIN alobListCriteriaAlob     AS c ON c.iPattern  = r.iPattern
                                          AND c.iCriteria = r.iCriteria
  INNER JOIN alobListPattern          AS p ON p.iPattern  = c.iPatternRelated
  WHERE 
           r.bSeed = 1               -- Only use the seed rows
--       AND r.iClassRelated <> p.iClass -- Do not insert cid of your related cid 
       AND NOT EXISTS ( SELECT * FROM #resultCriteria AS rcDup1 -- Do not insert n.ClassRelated, n.iPattern where 
                                                                --  values that already exist in #resultCriteria
                                                                --   for your iClassStart, iClassObject
                          WHERE 
       rcDup1.iClass         = p.iClass
                               AND rcDup1.iPattern       = p.iPattern
                               AND rcDup1.iClassStart    = r.iClassStart
                               AND rcDup1.iObjectStart   = r.iObjectStart
                      )
       AND NOT EXISTS ( SELECT * FROM #resultCriteria AS dup -- do not insert data that already exists in #resultCriteria
                        WHERE
                                 dup.iClass        = r.iClass
                             AND dup.iObject       = r.iObject
                             AND dup.iPattern      = p.iPattern
                             AND dup.iClassRelated = p.iClass
                             AND dup.iClassStart   = r.iClassStart
                             AND dup.iObjectStart  = r.iObjectStart
                      )

/*
  -- HACK 
  PRINT 'SELECT * FROM #needed have iPattern (1) which points to iPatternRelated (2) get iPattern (2)'
  SELECT * FROM #needed
*/
--  SELECT r.iClass, r.iObject, c.iPattern, p.sPatternClass, p.sPatternAttribute
  -- alobListCriteria
  -- iCriteria   iPattern    iPatternRelated 
  -- ----------- ----------- --------------- 
  -- 1           1           any            <- This exists in #resultCriteria
  -- 1           2           1              <- This row will be added to #needed

  INSERT INTO #needed (   iClass,   iObject, iClassRelated,   iClassStart,   iObjectStart,   iPattern,   sNameAttribute )
  SELECT DISTINCT       r.iClass, r.iObject,      p.iClass, r.iClassStart, r.iObjectStart, p.iPattern, p.sNameAttribute 
  FROM 	     #resultCriteria          AS r 
  INNER JOIN alobListCriteriaAlob     AS c ON c.iPatternRelated = r.iPattern 
                                          AND c.iCriteria       = r.iCriteria
  INNER JOIN alobListPattern          AS p ON p.iPattern        = c.iPattern
  WHERE 
           r.bSeed = 1                       -- Only use the seed rows
--       AND (    ( r.iClass <> p.iClass )     -- Do not insert cid of your related cid 
--             OR (  
--                )
--           )
--       AND r.iClass <> p.iClass -- Do not insert cid of your related cid
       AND NOT EXISTS ( SELECT * FROM #resultCriteria AS rcDup1 -- Do not insert n.ClassRelated, n.iPattern where 
                                                                --  values that already exist in #resultCriteria
                                                                --   for your iClassStart, iClassObject
                          WHERE 
                                   rcDup1.iClass         = p.iClass
                               AND rcDup1.iPattern       = p.iPattern
                               AND rcDup1.iClassStart    = r.iClassStart
                               AND rcDup1.iObjectStart   = r.iObjectStart
                      )
       AND NOT EXISTS ( SELECT * FROM #needed AS n -- Do not insert data that already exists in #needed
                          WHERE 
                                   n.iClass         = r.iClass
                               AND n.iObject        = r.iObject
                               AND n.iClassRelated  = p.iClass
                               AND n.iClassStart    = r.iClassStart
                               AND n.iObjectStart   = r.iObjectStart
                               AND n.iPattern       = p.iPattern
                               AND n.sNameAttribute = p.sNameAttribute
                      )
       AND NOT EXISTS ( SELECT * FROM #resultCriteria AS dup -- do not insert data that already exists in #resultCriteria
                        WHERE
                                 dup.iCriteria     = r.iCriteria
                             AND dup.iPattern      = p.iPattern
                             AND dup.iClass        = r.iClass
                             AND dup.iObject       = r.iObject
                             AND dup.iClassRelated = p.iClass
                         AND dup.iClassStart   = r.iClassStart
                             AND dup.iObjectStart  = r.iObjectStart
                      )
       AND NOT EXISTS ( SELECT * FROM #resultCriteria AS dup2 -- do not insert data that already exists in #resultCriteria
                        WHERE
                                 dup2.iCriteria      = r.iCriteria
                             AND dup2.iPattern       = p.iPattern
                             AND dup2.iClass         = p.iClass
                             AND dup2.iClassRelated  = r.iClass
                             AND dup2.iObjectRelated = r.iObject
                             AND dup2.iClassStart    = r.iClassStart
                             AND dup2.iObjectStart   = r.iObjectStart
                      )

  /*
  -- HACK 
  PRINT 'SELECT * FROM #needed + have iPattern (1) which is pointed to by another iPattern (2)"s iPatternRelated (1)'
  SELECT * FROM #needed
  */

  IF EXISTS ( SELECT * FROM #needed )
  BEGIN
-- INSERT INTO #data descendents of iClass, iObject as seen in ObjPathCache
    IF @incStart = 1
    BEGIN
      -- If @incStart is on (=1), then include descendents found in ObjPathCache only if the Starting Object 
      --   is also in the path.  This improves performance by not including objects in the CriteriaToPattern
      --   analysis which will not satisfy the criteria rules.  This can be used if the criteria includes
      --   patterns that are only linked in ascending order.  The @incStart option should be 0 (the default)
      --   when the criteria rules include analysis of siblings.  An example of a criteria without siblings is
      --
      --   CriteriaToPattern
      --   iCriteria   iPattern    iPatternRelated 
      --   ----------- ----------- ---------------
      --   88170       83           90
      --   88170       90           90
      --   88170       80717        90
      --   
      --   where 83 is a pattern for NLOC, 90 a pattern for NTSR, and 80717 a pattern for G02U
      -- When we analyze the "leaf" pattern, G02U object, then we analyze the ObjPathCache to find paths
      --   where an NTSR object is linked to that G02U object, and then the NLOC object linked to that NTSR
      --   object.  Once we have found the NLOC object, we look in ObjPathCache for NTSR objects 
      --   that are related to the NLOC object (because the Criteria says 90 is a related Pattern to 83).
      --   However, the CriteriaToPattern rules are only concerned with G02U objects that are linked
      --   to an NTSR.  There are no other criteria rules for NTSR as a parent. Hence there is no
      --   need to analyze all NTSR objects linked to this NLOC object.  The ABS config file can indicate this
      --   relationship by marking the PatternUsage of 88170 with "INCSO".  Then we can add a check
      --   to search for the G02U object (the starting object) in the NTSR objects path. 
      --   This significanly helps performance by reducing the number of NTSR entries to analyze when 
      --   an abstest command results in a large number of starting objects to analyze, and there are 
      --   lots of NTSR objects linked to the NLOC object. 
      --
      -- An example of a criteria with siblings is 
      --
      --   CriteriaToPattern
      --   iCriteria   iPattern    iPatternRelated 
      --   ----------- ----------- ---------------
      --   88170       83           90
      --   88170       90           90
      --   88170       80718        90
      --   88170       80717        90
      --   
      -- The criteria includes a rule with a pattern relationship 80718 related to 90 where
      --   80718 is a pattern for another class/attribute pattern.  To find all the relationships for
      --   this criteria requires us to analyze the NTSR objects related to the NLOC we found.
      --   Once we have those NTSR objects, then we need to find the objects for the 80718 pattern.
      --   The patterns 80718 and 80717 are siblings which are linked to the same parent (pattern 90).
      --
      INSERT INTO #data ( iClass, iObject,   iPattern,  sNameAttribute, sValueAttributeDeleted, sValueAttributeInserted,
                           iClassRelated, iObjectRelated,  iClassStart,    iObjectStart ) 
      SELECT DISTINCT      d.cid,    d.id, n.iPattern, n.sNameAttribute,                  NULL,                    NULL,
                                n.iClass,      n.iObject, n.iClassStart, n.iObjectStart
      FROM #needed AS n
      INNER JOIN ObjPathCache         AS o ON o.cid             =    n.iClass
                                          AND o.id              =    n.iObject
      INNER JOIN ObjPathCache         AS d ON d.idpath          LIKE o.idpath + '/%'
                                          AND d.cid             =    n.iClassRelated
      -- Extra join to filter for those paths that include the Starting Object
      INNER JOIN ObjPathCache         AS st ON st.idpath        LIKE d.idpath + '/%'            
                                          AND st.cid            =    n.iClassStart        
                                          AND st.id             =    n.iObjectStart           
  
      WHERE
               NOT EXISTS ( SELECT * FROM #resultCriteria AS rcDup1
                            WHERE
                                     rcDup1.iPattern       = n.iPattern
                                 AND rcDup1.iClass         = d.cid
                                 AND rcDup1.iObject        = d.id
                                 AND rcDup1.iClassRelated  = n.iClass
                                 AND rcDup1.iObjectRelated = n.iObject
                                 AND rcDup1.iClassStart    = n.iClassStart
                                 AND rcDup1.iObjectStart   = n.iObjectStart
                          )
           AND NOT EXISTS ( SELECT * FROM #resultCriteria AS rcDup2
                            WHERE
                                     rcDup2.iPattern       = n.iPattern
                                 AND rcDup2.iClass         = n.iClass
                                 AND rcDup2.iObject        = n.iObject
                                 AND rcDup2.iClassRelated  = d.cid
                                 AND rcDup2.iObjectRelated = d.id
                                 AND rcDup2.iClassStart    = n.iClassStart
                                 AND rcDup2.iObjectStart   = n.iObjectStart
                          )
    END
    ELSE
    BEGIN
      -- INSERT INTO #data descendents of iClass, iObject as seen in ObjPathCache regardless if Starting Object in the path
      INSERT INTO #data ( iClass, iObject,   iPattern,  sNameAttribute, sValueAttributeDeleted, sValueAttributeInserted,
                           iClassRelated, iObjectRelated,  iClassStart,    iObjectStart ) 
      SELECT DISTINCT      d.cid,    d.id, n.iPattern, n.sNameAttribute,                  NULL,                    NULL,
                                n.iClass,      n.iObject, n.iClassStart, n.iObjectStart
      FROM #needed AS n
      INNER JOIN ObjPathCache         AS o ON o.cid             =    n.iClass
                                          AND o.id              =    n.iObject
      INNER JOIN ObjPathCache         AS d ON d.idpath          LIKE o.idpath + '/%'
                                          AND d.cid             =    n.iClassRelated

      WHERE
               NOT EXISTS ( SELECT * FROM #resultCriteria AS rcDup1
                            WHERE
                                     rcDup1.iPattern       = n.iPattern
                                 AND rcDup1.iClass         = d.cid
                                 AND rcDup1.iObject        = d.id
                                 AND rcDup1.iClassRelated  = n.iClass
                                 AND rcDup1.iObjectRelated = n.iObject
                            AND rcDup1.iClassStart    = n.iClassStart
                                 AND rcDup1.iObjectStart   = n.iObjectStart
                          )
           AND NOT EXISTS ( SELECT * FROM #resultCriteria AS rcDup2
                            WHERE
                                     rcDup2.iPattern       = n.iPattern
                                 AND rcDup2.iClass         = n.iClass
                                 AND rcDup2.iObject        = n.iObject
                                 AND rcDup2.iClassRelated  = d.cid
                                 AND rcDup2.iObjectRelated = d.id
                                 AND rcDup2.iClassStart    = n.iClassStart
                                 AND rcDup2.iObjectStart   = n.iObjectStart
                          )
    END

    /*     
    -- HACK
    PRINT 'SELECT * FROM #data -- after descendent population'
    SELECT * FROM #data
    */

    CREATE INDEX tneededIndex2 ON #needed(iClass,iObject)
    --CREATE INDEX tresultCriteria2 ON #resultCriteria(iClass,iObject,bSeed)

    EXEC alobGetRelatedByClassSub
  END
  /*
  -- HACK
  PRINT 'SELECT * FROM #data -- after ascendent population'
  SELECT * FROM #data
  */

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN alobGetRelatedByClass

GO

