-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/alobprocessevqueueobject.sql
-- SID: 1.9.4.8
-- Date Modified: 07/09/11 05:41:44
--
-- Module Description
--   Defines the alobProcessEvQueueObject stored procedure

Use Object
GO

-------------------------------------------------------------------------
-- alobProcessEvQueueObject
PRINT 'Creating procedure alobProcessEvQueueObject'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobProcessEvQueueObject' AND type = 'P')
  DROP PROCEDURE alobProcessEvQueueObject
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/alobprocessevqueueobject.sql
-- SID: 1.9.4.8
-- Date Modified: 07/09/11 05:41:44
CREATE PROCEDURE alobProcessEvQueueObject
  @bManual      INTEGER         = 0   , -- 0 to use the trigger populated table, 1 to use the manually populated table
  @iPattern     INTEGER         = NULL, -- if @bManual = 1 this can be used to limit the LOBs created
  @iCriteria    INTEGER         = NULL, -- if @bManual = 1 this can be used to limit the LOBs created
  @sNamePathLob NVARCHAR( 256 ) = NULL, -- if @bManual = 1 this can be used to limit the LOBs created
  @bDisplayMethods TINYINT      = 0,    -- display method invocation info
  @iDetail      INTEGER         = 0     -- display BuildBusinessSystem resource details
AS

  -- CAUTION: There are no transaction logic statements in this procedure; here's why:
  --          This stored procedure performs as an outer loop to alobProcessEvQueueObjectDetail.
  --          This inner stored procedure uses multiple serial transactions to reduce potential
  --          locks or blocks to smaller lengths of time.  If this inner stored procedure was
  --          wrapped in a single transaction this mechanism would be defeated.

  SET NOCOUNT ON  -- Turning NOCOUNT ON to reduce output in the Scheduled Job.
                  -- If a scheduled job has SET NOCOUNT OFF it will take significantly longer.
                  -- In this case, seven times longer.

  DECLARE @bDone INTEGER
  DECLARE @dCreate DATETIME

  SELECT  @bDone = 0  -- Assume that it is not done

  WHILE ( 0 = @bDone ) -- Not done, so do some processing
  BEGIN
    IF ( 0 = @bManual ) -- Check the automatic queue for work
    BEGIN
      SELECT @dCreate =  MIN(dCreate) FROM evQueueObject

      IF @dCreate IS NULL
        SELECT @bDone = 1 -- There is nothing there, so there is nothing to do, set done to 1 ( true )
      ELSE
      BEGIN
        IF NOT EXISTS ( SELECT 1 FROM ObjPathCacheRun WHERE @dCreate < StartTime AND EndTime IS NOT NULL )
        BEGIN
          SELECT @bDone = 1 -- There is nothing there, so there is nothing to do, set done to 1 ( true )
        END
      END

      IF @bDone = 0
      BEGIN
        --  Update the evQueueObject.  The evQueueObject table is updated by object INSERT
        --  and UPDATE during object creation by cl_process<cidL>.  This creates problems
        --  for ABS if the user has defined a configuration to look for NULL and not NULL
        --  values.  The evQueueObject will be updated to only include the UPDATE entry
        --  which contains all the information for ABS creation. 
        DELETE evQueueObject
          FROM (SELECT MAX(event_number) AS maxEvent, cid, id, attribute_name FROM evQueueObject
            GROUP BY event_number, cid, id, attribute_name) AS maxTab
          WHERE evQueueObject.event_number < maxTab.maxEvent
            AND evQueueObject.cid = maxTab.cid AND evQueueObject.id = maxTab.id 
            AND evQueueObject.attribute_name = maxTab.attribute_name

      END
    END
    ELSE -- Check the manual queue for work
    BEGIN
      IF NOT EXISTS ( SELECT * 
                      FROM evQueueObjectManual AS q
                    )
      BEGIN
        SELECT @bDone = 1 -- There is nothing there, so there is nothing to do, set done to 1 ( true )
      END
    END
    IF ( 0 = @bDone ) -- Still not done, do some processing
    BEGIN
      EXECUTE alobProcessEvQueueObjectDetail 
        @bManual         = @bManual, 
        @iPattern        = @iPattern,
        @iCriteria       = @iCriteria, 
        @sNamePathLob    = @sNamePathLob,
        @bDisplayMethods = @bDisplayMethods,
        @iDetail         = @iDetail
    END
--    WAITFOR DELAY '000:00:10' -- Give other processes a chance
--    SELECT @bDone = 1
  END
GO
