-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup_password.sql, tbsm_base, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/09/28 07:54:09
--
-- Module Description
--   Defines the tep_setup_password stored procedure

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tep_setup_password' AND type = 'P')
  DROP PROCEDURE tep_setup_password
GO

PRINT 'Creating procedure tep_setup_password'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup_password.sql, tbsm_base, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/09/28 07:54:09
-- tep_setup_password is invoked from tep_setup to change TBSM TEP configuration passwords
CREATE PROCEDURE tep_setup_password
  @soap_server   ObjName      = NULL,      -- TEMS Soap Server tcp/ip name or address
  @soap_alias    NVARCHAR(20) = NULL,      -- TEMS Soap Server Alias name.  Default is 'soap'.  If a TEMS SOAP Server Hub is defined, specify the ALIAS here (see install doc).
  @soap_uid      NVARCHAR(20) = NULL,      -- (Optional) Userid to be used to query TEMS via SOAP.
  @soap_pw       NVARCHAR(20) = NULL,      -- (Optional) Password for the @soap_uid		
  @teps_server   ObjName      = NULL,      -- TEPS tcp/ip name or address.  Require for
  @teps_uid	 NVARCHAR(20) = NULL,      -- Userid to be used to read TEPS tables
  @teps_pw	 NVARCHAR(20) = NULL,      -- Password for @teps_uid. Note: Case sensitive.
  @verbose       TINYINT = 0
AS

  SET ANSI_DEFAULTS OFF
  SET CONCAT_NULL_YIELDS_NULL OFF
  SET NOCOUNT ON

  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT

  DECLARE @rc INT, @err INT, @sql NVARCHAR(4000), @start DATETIME


  DECLARE @anchor_physical ObjName, @anchor_business ObjName
  DECLARE @linked_server NVARCHAR(100), @teps_from_clause NVARCHAR(100), @teps_provider NVARCHAR(20)
  DECLARE @enabled TINYINT, @refresh_excp_id ObjID
  DECLARE @job_name sysname, @job_id UNIQUEIDENTIFIER, @ret INT, @job_status NVARCHAR(20)
  DECLARE @soap_url NVARCHAR(100), @soap_event_laststamp NVARCHAR(20)
  DECLARE @srvid INT, @srvstatus INT, @providername ObjName
  DECLARE @soap_request NVARCHAR(1000), @output_error NVARCHAR(1000), @obj_id ObjID
  DECLARE @wait INT

--  CREATE TABLE #RLSESTAMP (RLSESTAMP NVARCHAR(50) NULL)
--  CREATE TABLE #Name (Name NVARCHAR(64) NULL)
--  CREATE TABLE #cmdout (line NVARCHAR(500) NULL, rowno INT IDENTITY(1,1) )

------------------------------------------------------------------------------------
--  
------------------------------------------------------------------------------------

  IF ISNULL(@soap_uid,'') <> '' 
  AND NOT EXISTS(SELECT * FROM tep_config 
                   WHERE soap_uid = @soap_uid 
                     AND soap_server = ISNULL(@soap_server,soap_server) 
                     AND soap_alias  = ISNULL(@soap_alias, soap_alias) )
  BEGIN
    RAISERROR('No entries found in tep_config for soap_uid %s %s %s.  Use tep_setup ''LIST'' to see current configuration ',3,1, @soap_uid, @soap_server, @soap_alias) WITH NOWAIT
    RETURN -30000
  END
  IF ISNULL(@teps_uid,'') <> '' 
  AND NOT EXISTS(SELECT * FROM tep_config 
                   WHERE teps_uid = @teps_uid
                   AND teps_server = ISNULL(@teps_server,teps_server) )
  BEGIN
    RAISERROR('No entries found in tep_config for teps_uid %s %s.  Use tep_setup ''LIST'' to see current configuration ',3,1, @teps_uid, @teps_server) WITH NOWAIT
    RETURN -30000
  END

  IF ISNULL(@soap_uid,'') <> '' 
  BEGIN
    DECLARE cur_config INSENSITIVE CURSOR FOR
      SELECT DISTINCT soap_server, soap_alias
        FROM tep_config
        WHERE soap_uid = @soap_uid
          AND soap_server = ISNULL(@soap_server,soap_server)
          AND soap_alias  = ISNULL(@soap_alias, soap_alias)
        ORDER BY soap_server, soap_alias
    OPEN cur_config
    FETCH NEXT FROM cur_config INTO @soap_server, @soap_alias
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC tep_setup_add_soap_hdr @soap_server=@soap_server, @soap_alias=@soap_alias, @soap_uid=@soap_uid, @soap_pw=@soap_pw
      RAISERROR('Changed soap_pw for soap_uid %s, soap_server %s, soap_alias %s.',0,1, @soap_uid, @soap_server, @soap_alias) WITH NOWAIT

      FETCH NEXT FROM cur_config INTO @soap_server, @soap_alias
    END
    DEALLOCATE cur_config
  END

  IF ISNULL(@teps_uid,'') <> '' 
  BEGIN
    DECLARE cur_config INSENSITIVE CURSOR FOR
      SELECT DISTINCT teps_server
        FROM tep_config
        WHERE teps_uid = @teps_uid
          AND teps_server = ISNULL(@teps_server,teps_server)
        ORDER BY teps_server
    OPEN cur_config
    FETCH NEXT FROM cur_config INTO @teps_server
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      SET @linked_server = 'teps_'+@teps_server

      EXEC @rc = sp_droplinkedsrvlogin  @rmtsrvname=@linked_server, @locallogin=NULL
      EXEC @rc = sp_addlinkedsrvlogin   @rmtsrvname=@linked_server, @useself='false', @rmtuser=@teps_uid, @rmtpassword=@teps_pw
      IF @rc <> 0
      BEGIN
        RAISERROR('sp_addlinkedsrvlogin FAILED for %s',3,1,@linked_server) WITH NOWAIT
        DEALLOCATE cur_config
        RETURN -30000
      END
      RAISERROR('Changed teps_pw for teps_uid %s, teps_server %s.',0,1, @teps_uid, @teps_server) WITH NOWAIT

      FETCH NEXT FROM cur_config INTO @teps_server
    END
    DEALLOCATE cur_config
  END

GO





